clear;clc

fprintf('\t Create correlation report: %s\n',datetime('now'));

%% session specifc input files
% table of qcodes 
f_qcodetable = 'psfile.out';

% snr ratios
f_snr = 'snr.txt';

% control file
f_fourfitcontrol = 'aua058.cf';

% clk section
f_clk = 'clk.correp';

%% output files
% name of correlator report
f_correp = 'aua058.correp';

%% input data
% stations processed (two letter codes)
stations = {'Ht','Ho','Wa','Ww'};

% name of correlator
CORREL = 'VIEN';

% name of database
DATABASE = '19NOV28XA';

% session name
SESSNAME = 'AUA058'; 

% observation time
OBSTIME = '2019/11/28 2019/11/29';

% start time of session
UTSTART = '14:00';

% duration of session
DURATION = '24'; 

% doy of session
DOY = '332-333';

% time of correlation
CORRTIME = '2020/05/05';

% correlator notes
CORRELATOR_NOTES = 'Wa X-band only. Ho and Ke did not observe. No fringes for Yg. Some of the observed sources are without any flux information, therefore the predicted SNR is not available. For the creation of the +SNR_RATIOS table the averaged observed SNR of all baselines for X/S separately have been used.';

% string for closing
END1 = 'Best Regards,';
END2 = 'Jakob Gruber';

%% db input files
% legend of qcodes
f_qclegend = 'qcode_legend.txt';

% station name database
f_nscodes = 'ns-codes.txt';

% hops one letter code
f_hopslettercode = 'hops_station_code.txt';

%% config
% session type
sess_type = 'S/X';

%% nothing to change below here
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          %%          %%          %%          %%          %%     %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% add paths
addpath('db/')
addpath('input/')
addpath('lib/')

%% read in station database
% save to cell
Cns = read_nscodestxt_file(f_nscodes);

%% read in hops station code
% save to cell 
Chs = read_hops_station_code_file(f_hopslettercode);

%% read in fourfit control file
% save to struct
cf = parse_fourfit_control_file(f_fourfitcontrol);

%% define fourfit frequency channel labeling
% check mode
if strcmp(sess_type,'S/X')
    [freqlabel,freqname] = fourfit_channel_label;
end

%% create ST cell
% cell with station attributes and values
% sorted by column:
%   1: station name 8 character (ivs site name)
%   2: two letter station code
%   3: hops 1 letter code
%   4: station notes
%   5: string of dropped channels
%   6: manual phase-cal applied

% number of stations
Nst = length(stations);

% preallocate cell
ST = cell(Nst,6);

% create cell
for iSt = 1:Nst
    % store into temporary variable
    ivs_2lc = stations{iSt};
    
    % find ivs site name in ns-codes.txt cell
    ivs_sn = Cns{1,2}{strcmp(Cns{1,1},ivs_2lc),1};
    
    % store ivs site name
    ST{iSt,1} = ivs_sn;
    
    % store ivs 2 letter 
    ST{iSt,2} = ivs_2lc;
    
    % store hops 1 letter code
    ST{iSt,3} = Chs{1,2}{strcmp(Chs{1,1},ivs_2lc),1};
    
    % default value to comments
    ST{iSt,4} = '<comment here>';
    
    %% dropped channels    
    % get dropped channels in correct correlator report string format
    [DROP_NAME] = final_dropped_channels(cf, ST{iSt,3}, freqlabel, freqname);
        
    % if there freq names to write to struct
    if ~isempty(DROP_NAME)
        ST{iSt,5} = DROP_NAME;
    end
    
    %% manual phase cal applied
    % check if manual phase cal is applied for stations
    [o] = grep_pc_mode_manual(cf, ST{iSt,3});
    
    % store pc_mode manual search result to ST
    ST{iSt,6} = o;
    
end

%% hardcoded
% hardware_correlator = 0, software_correlator = 1;
CORRPASS = '1';

% indicates whether the databases have been submitted to the IVS data centers
EXPORT = 'DONE';

%% qcodes summary
% read in qcode table
qcodes = fileread(f_qcodetable);

% read in qcode legend
qcode_legend = fileread(f_qclegend); 

% retrieve qcode totals [char]
qcodes_totals_str = qcodes(strfind(qcodes,'Totals'):end);

% char to cell
qcodes_totals_cell = textscan(qcodes_totals_str,'%s');

% qcodd totals [double]
qcs = str2double(qcodes_totals_cell{1}(2:end))';

%% clock file
% read in clock file in correlation report format
cf_clk = fileread(f_clk);

%% fourfit control file
% read in fourfit control file
cf_1234 = fileread(f_fourfitcontrol);

fprintf('\t Input data is set\n')

%% calcs

%% Calculation SUMMARY
% number of observations correlated:
n = sum(qcs(1:19));

% number of observations with qcodes 5-9
nqgood = sum(qcs(6:10));
pqgood = nqgood/n*100;

% number of observations with qcodes 0
nqbad = qcs(1);
pqbad = nqbad/n*100;

% number of obserations with qcodes 4-1,A-H,N
nqprobl = sum(qcs([2:5,11:19]));
pqprobl = nqprobl/n*100;

%% Calculation for SNR ratios
fid = fopen(f_snr);
il = 1;
tline = fgetl(fid);
while ischar(tline)
    if ~strcmp(tline(1),'*')
        strs = strsplit(tline);
        istr1 = strfind(strs{1},'.X.');
        if ~isempty(istr1)
            cbl = strs{1}(istr1-2:istr1-1);
            cb = 'X';
        end

        istr1 = strfind(strs{1},'.S.');
        if ~isempty(istr1)
            cbl = strs{1}(istr1-2:istr1-1);
            cb = 'S';
        end

        B_obs{il,1} = cb;
        BL_obs{il,1} = cbl;
        csnr = strs{5};
        SNR_obs(il,1)=str2num(csnr);
        il=il+1;
    end
    tline = fgetl(fid);
end
fclose(fid);
NBL = unique(BL_obs);
NB = unique(B_obs);
idata = 1;
for iBL = 1:length(NBL)
    istr1 = find(strcmp(NBL{iBL},BL_obs));
    for iB = 1:length(NB)
        istr2 = find(strcmp(NB{iB},B_obs));
        istr = intersect(istr1,istr2);
        if strcmp(NB{iB},'X')
            meanObs_BL_X{iBL,1} = NBL{iBL};
            meanObs_Bnd_X{iBL,1} = NB{iB};
            m_X(iBL,1) = mean(SNR_obs(istr));
            nm_X(iBL,1) = length(SNR_obs(istr));
        end
        if strcmp(NB{iB},'S')
            meanObs_BL_S{iBL,1} = NBL{iBL};
            meanObs_Bnd_S{iBL,1} = NB{iB};
            m_S(iBL,1) = mean(SNR_obs(istr));
            nm_S(iBL,1) = length(SNR_obs(istr));
        end  
%             idata = idata + 1;
    end   
end

predicted_SNR_X = mean(m_X);
predicted_SNR_S = nanmean(m_S);

meanRatio_SNR_X = m_X/predicted_SNR_X;
meanRatio_SNR_S = m_S/predicted_SNR_S;

fprintf('\t Calculations are finished\n')

%% Write to file
fileID = fopen(f_correp,'w'); 
% +HEADER
fprintf(fileID,'+HEADER\n');
fprintf(fileID,'CORREL   %s\n',CORREL);
fprintf(fileID,'DATABASE %s\n',DATABASE);
fprintf(fileID,'SESSNAME %s\n',SESSNAME);
fprintf(fileID,'OBSTIME  %s\n',OBSTIME);
fprintf(fileID,'UTSTART  %s\n',UTSTART);
fprintf(fileID,'DURATION %s\n',DURATION);
fprintf(fileID,'DOY      %s\n',DOY);
fprintf(fileID,'CORRTIME %s\n',CORRTIME);
fprintf(fileID,'CORRPASS %s\n',CORRPASS);
fprintf(fileID,'EXPORT   %s\n\n',EXPORT);
% +SUMMARY
fprintf(fileID,'+SUMMARY\n');
fprintf(fileID,' Qcode      %% of Total   %% of Correlated\n');
fprintf(fileID,'                scans          scans\n\n');
fprintf(fileID,'  5-9           %3.0f%%             -\n',pqgood);
fprintf(fileID,'   0            %3.0f%%             -\n',pqbad);
fprintf(fileID,' 4-1,A-H,N      %3.0f%%             -\n',pqprobl);
fprintf(fileID,' Removed           -              -\n\n');
% +CORRELATOR_NOTES
fprintf(fileID,'+CORRELATOR_NOTES\n\n');
fprintf(fileID,'%s\n\n',CORRELATOR_NOTES);
% +STATION_NOTES
fprintf(fileID,'+STATION_NOTES\n\n');
for i = 1:length(ST(:,1))
    fprintf(fileID,'  %s (%s/%s): %s\n\n',ST{i,1},ST{i,2},ST{i,3},ST{i,4});
end
% +DROP_CHANNELS
fprintf(fileID,'+DROP_CHANNELS\n');
for i = 1:length(ST(:,1))
    fprintf(fileID,'  %s %s\n',ST{i,2}, ST{i,5});
end
% +MANUAL PCAL
fprintf(fileID,'\n+MANUAL PCAL ');
for i = 1:length(ST(:,1))
    if ST{i,6} == 1
        fprintf(fileID,' %s',ST{i,2});
    end
end

% +CHANNELS
fprintf(fileID,'\n\n+CHANNELS\n');
fprintf(fileID,' Channels: XR1U = band|polarization|channel#|sideband\n\n');
fprintf(fileID,'XR1U/L  BBC01 8214 \n');
fprintf(fileID,'XR2U    BBC02 8230 \n');
fprintf(fileID,'XR3U    BBC03 8470 \n');
fprintf(fileID,'XR4U    BBC04 8486 \n');
fprintf(fileID,'XR5U    BBC05 8774 \n');
fprintf(fileID,'XR6U    BBC06 8838 \n');
fprintf(fileID,'XR7U    BBC07 8902 \n');
fprintf(fileID,'XR8U/L  BBC08 8934 \n');
fprintf(fileID,'SR1U    BBC09 2200 \n');
fprintf(fileID,'SR2U    BBC10 2216 \n');
fprintf(fileID,'SR3U    BBC11 2232 \n');
fprintf(fileID,'SR4U    BBC12 2256 \n');
fprintf(fileID,'SR5U    BBC13 2272 \n');
fprintf(fileID,'SR6U    BBC14 2288 \n');

warning('Channel frequencies are hardcoded')

% +CLOCKS
fprintf(fileID,'\n');
fprintf(fileID,cf_clk);
fprintf(fileID,'\n');

% +QCODES
fprintf(fileID,'+QCODES\n\n');
fprintf(fileID,qcodes);
fprintf(fileID,'\n');
fprintf(fileID,qcode_legend);

% +SNR_RATIOS
fprintf(fileID,'\n');
fprintf(fileID,'+SNR_RATIOS\n\n');
fprintf(fileID,'MEAN RATIOS = Observed SNR / Predicted SNR \n\n');
fprintf(fileID,' ...by baseline, over all sources:\n\n');
fprintf(fileID,' bl     X     n       S     n\n\n');
for i_bl = 1:length(meanRatio_SNR_X)
    if isnan(meanRatio_SNR_X(i_bl,1))
        fprintf(fileID,' %s    %s  %3.0f     %1.2f  %3.0f\n',meanObs_BL_X{i_bl,1},'-',nm_X(i_bl),meanRatio_SNR_S(i_bl),nm_S(i_bl));
        continue;
    end
    if isnan(meanRatio_SNR_S(i_bl,1))
        fprintf(fileID,' %s    %1.2f  %3.0f      %s   %3.0f\n',meanObs_BL_X{i_bl,1},meanRatio_SNR_X(i_bl),nm_X(i_bl),'-',nm_S(i_bl));
        continue;
    end    
    fprintf(fileID,' %s    %1.2f  %3.0f     %1.2f  %3.0f\n',meanObs_BL_X{i_bl,1},meanRatio_SNR_X(i_bl),nm_X(i_bl),meanRatio_SNR_S(i_bl),nm_S(i_bl));
end

% +FOURFIT_CONTROL_FILE
fprintf(fileID,'\n\n+FOURFIT_CONTROL_FILE\n\n');
fprintf(fileID,cf_1234);
% fprintf(fileID,cf_1234_ww);
% fprintf(fileID,cf_run);

% +END
fprintf(fileID,'\n');
fprintf(fileID,'%s\n',END1);
fprintf(fileID,'%s\n\n',END2);
fprintf(fileID,'+END');
fclose(fileID);

fprintf('\t Congratulation! You created a correlation report!\n')
fprintf('\n')
fprintf('\t\tYou can find the correlation report in: %s\n', f_correp)